if(!isObject(ZG_MazeGen))
{
	new ScriptObject(ZG_MazeGen) { };
}

function ZG_MazeGen::init(%this)
{
	echo("initializing maze");

	for(%i = 0; %i < 25; %i++)
	{
		for(%j = 0; %j < 25; %j++)
		{
			%this.setStaticTile(%i, %j, SquareBordersImageMap, 15);
			%this.setTileCustomData(%i, %j, "15 0");			
		}		
	}
	
	echo("maze initialized");
}

function ZG_MazeGen::generateMaze(%this)
{
	$generateStartTime = getRealTime();
	echo("Generating Maze");

	%maxX = 25;
	%maxY = 25;

	while(%this.visitedCells < %this.totalCells)
	{
		%numIntactNeighbors = 0;

		%curX = getWord(%this.currentCell, 0);
		%curY = getWord(%this.currentCell, 1);
		
		//find all neighbors of CurrentCell with all walls intact
		//making sure not to overflow the tilemap
		if( %curX > 0 )
		{
			if( %this.walls( %curX - 1, %curY ) == 15 )
			{
				%intactNeighbors[%numIntactNeighbors] = %curX - 1 SPC %curY;
				%numIntactNeighbors++;
			}
		}

		if( %curX < %maxX - 1 )
		{
			if( %this.walls( %curX + 1, %curY ) == 15 )
			{
				%intactNeighbors[%numIntactNeighbors] = %curX + 1 SPC %curY;
				%numIntactNeighbors++;
			}
		}

		if( %curY > 0 )
		{
			if( %this.walls( %curX, %curY - 1 ) == 15 )
			{
				%intactNeighbors[%numIntactNeighbors] = %curX SPC %curY - 1;
				%numIntactNeighbors++;
			}
		}

		if( %curY < %maxY - 1 )
		{
			if( %this.walls( %curX, %curY + 1 ) == 15 )
			{
			  %intactNeighbors[%numIntactNeighbors] = %curX SPC %curY + 1;
			  %numIntactNeighbors++;
			}
		}
		
		//if one or more found 
		if( %numIntactNeighbors > 0 )
		{
			//choose one at random 
			%newCellIndex = getRandom( %numIntactNeighbors - 1 );
			%tempCell = %intactNeighbors[%newCellIndex];
				  
			//knock down the wall between it and CurrentCell 
			%this.knockDownWalls(%this.currentCell, %tempCell);
			
			//push CurrentCell location on the CellStack  
			%this.cellStack.push(%this.currentCell);
			
			//make the new cell CurrentCell  
			%this.currentCell = %tempCell;
			
			//add 1 to VisitedCells
			%this.visitedCells++;
		} 
		else
		{
			%this.currentCell = %this.cellStack.pop();
		}
	}	
	
	echo("Generated in" SPC getRealTime() - $generateStartTime SPC "ms.");  
	%this.updateBorders();
}

function ZG_MazeGen::knockDownWalls(%this, %cell1, %cell2)
{	
	%cell1X = getWord(%cell1, 0);
	%cell1Y = getWord(%cell1, 1);
	%cell2X = getWord(%cell2, 0);
	%cell2Y = getWord(%cell2, 1);
	
	%cell1TempData = %this.walls(%cell1X, %cell1Y);
	%cell2TempData = %this.walls(%cell2X, %cell2Y);
		  
	if(%cell1X - %cell2X < 0 && %cell1Y - %cell2Y == 0 ) //tile to the right
	{  
		%this.setTileCustomData(%cell1X, %cell1Y, %this.walls(%cell1X, %cell1Y) - 2 );
		%this.setTileCustomData(%cell2X, %cell2Y, %this.walls(%cell2X, %cell2Y) - 8 );		  
	}
	
	if(%cell1X - %cell2X > 0 && %cell1Y - %cell2Y == 0 ) //tile to the left
	{
		%this.setTileCustomData(%cell1X, %cell1Y, %this.walls(%cell1X, %cell1Y) - 8 );
		%this.setTileCustomData(%cell2X, %cell2Y, %this.walls(%cell2X, %cell2Y) - 2 ); 
	}
	
	if(%cell1X - %cell2X == 0 && %cell1Y - %cell2Y < 0 ) //tile above
	{
		%this.setTileCustomData(%cell1X, %cell1Y, %this.walls(%cell1X, %cell1Y) - 4 );
		%this.setTileCustomData(%cell2X, %cell2Y, %this.walls(%cell2X, %cell2Y) - 1 ); 
	}
	
	if(%cell1X - %cell2X == 0 && %cell1Y - %cell2Y > 0 ) //tile below
	{
		%this.setTileCustomData(%cell1X, %cell1Y, %this.walls(%cell1X, %cell1Y) - 1 );
		%this.setTileCustomData(%cell2X, %cell2Y, %this.walls(%cell2X, %cell2Y) - 4 ); 
	}
}

function ZG_MazeGen::walls( %this, %posX, %posY )
{
	return getWord(%this.getTileCustomData(%posX, %posY), 0);
}

function ZG_MazeGen::updateBorders( %this )
{
  %maxX = 25;
  %maxY = 25; 

  for( %x = 0; %x < %maxX; %x++ )
  {
	 for( %y = 0; %y < %maxY; %y++ )
	 {	 
		%walls = %this.walls( %x, %y );
		
		%this.setStaticTile( %x, %y, SquareBordersImageMap, %walls );
	 }
  }
  %goal = %maxX - 1 SPC %maxY - 1;
  %this.schedule(150, solveMaze, "0 0", %goal);
}

function ZG_MazeGen::markVisited(%this, %x, %y)
{  
	%this.setTileCustomData(%x, %y, 1);
}

function ZG_MazeGen::visited(%this, %x, %y)
{  
	if(%this.solved.getTileCustomData(%x, %y) == 1)
	{
		return true;
	}
	else
	{
		return false;
	}
}